use crate::{commands::utils, Context, Error};

pub mod eurosport;

#[derive(Debug, poise::SlashChoiceParameter)]
pub enum Timeframe {
    #[name = "Currently happening"]
    Current,
    #[name = "Currently happening or starting in the future"]
    Future,
    #[name = "Currently happening or already ended"]
    Past,
    #[name = "Everything"]
    Everything,
}

#[derive(Debug, poise::SlashChoiceParameter)]
pub enum Source {
    #[name = "Get links for the Eurosport player"]
    Eurosport,
    #[name = "Get links for the apocalympics Eurosport player"]
    Olympics,
}

/// Get links for high tier commands.
#[poise::command(slash_command)]
pub async fn links(
    ctx: Context<'_>,
    #[description = "Where to git the juicy links from?"] source: Source,
    #[description = "Filter sessions for when they are/were happening, defaults to future"]
    timeframe: Option<Timeframe>,
    #[description = "Content to filter on"] filter: Option<String>,
) -> Result<(), Error> {
    if !utils::high_tier(ctx).await {
        ctx.say("This command can only be used in high tier channels for security")
            .await?;
        return Ok(());
    }

    match source {
        Source::Eurosport => eurosport::proc_eurosport(ctx, timeframe, filter).await,
        Source::Olympics => eurosport::proc_olympics(ctx, timeframe, filter).await,
    }

    // Ok(())
}
